/*---------------------------------------------------------------------------
 * 2005 NORSYS
 * main author : nono
 *
 * This software is a computer program whose purpose is to provide abstraction
 * for accessing directory data sources within java applications.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * Created on Oct 19, 2005
 * --------------------------------------------------------------------------*/
package speculoos.jndi;

import java.util.Map;

import speculoos.core.Mapper;
import speculoos.core.MapperException;


/**
 * This interface executes a modify operation on a JNDI/LDAP 
 * data source.
 * <p>
 * A modify operation is used to change, add or delete attributes 
 * of a <em>single</em> node in the underlying data source. 
 * </p>
 * @author nono
 * @version $Id: ModifyMapper.java 259 2006-05-23 10:34:50Z /C=FR/ST=Nord/L=Lille/O=Norsys SA/OU=UE/CN=Arnaud Bailly/emailAddress=abailly@norsys.fr $
 * @see <a href="/doc/conception/spec-api.html">API documentation</a>
 */
public interface ModifyMapper extends Mapper {

    /**
     * Execute a modify operation on a single node in the underlying
     * data source.
     * <p>
     * The <code>input</code> map is used as usual as variables substitution map
     * in the query and defines the attributes touched by the modify operation. 
     * This query contains several operations which may be update of values, adding 
     * of non existent attributes or removal of existing attributes.
     * </p>
     * <p>
     * This method returns nothing but if any operation fails, then a 
     * MapperException will be thrown containing the root cause of the 
     * failure (if any)
     * </p>
     * 
     * @param input a ModificationItem[] array.
     * @param env the local environment to use. May be null.
     * @return its input.
     * @throws MapperException if input map does not contains required 
     * parameters for the filter or if a validation rule is broken.
     */
    Object modify(Object input,Map env)throws MapperException;
}
