/*---------------------------------------------------------------------------
 * 2005 NORSYS
 * main author : nono
 *
 * This software is a computer program whose purpose is to provide abstraction
 * for accessing directory data sources within java applications.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * Created on Oct 19, 2005
 * --------------------------------------------------------------------------*/
package speculoos.jndi;

import java.util.Map;

import speculoos.core.Mapper;
import speculoos.core.MapperException;


/**
 * A mapper that allows renaming (ie. moving) of nodes.
 * <p>
 * This mapper allows one to move a node in the underlying directory 
 * tree from one parent to another parent. The node is not changed 
 * except that it is reparented according to the parameters of 
 * the configured query.
 * </p>
 * 
 * @author nono
 * @version $Id: RenameMapper.java 259 2006-05-23 10:34:50Z /C=FR/ST=Nord/L=Lille/O=Norsys SA/OU=UE/CN=Arnaud Bailly/emailAddress=abailly@norsys.fr $
 */
public interface RenameMapper extends Mapper {

    /**
     * Execute renaming of a node.
     * <p>
     * The input map is used only as variable susbtituiton for 
     * the content of the query.
     * </p>
     * 
     * @param input an object. May be null if no 
     * variable substitution occurs in the query.
     * @param env local environnement to customize the query. May not be 
     * null.
     * @return its input.
     * @throws MapperException if a variable is missing  in the input 
     * map for proper substitution or if something gets wrong in the
     * query.
     */
    Object rename(Object input,Map env)throws MapperException;
    
}
