/*---------------------------------------------------------------------------
 * ???2005 NORSYS
 * main author : jeff
 *
 * This software is a computer program whose purpose is to provide
 abstraction
 * for accessing directory data sources within java applications.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems
 and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * Created on 1 nov. 2005
 *
 --------------------------------------------------------------------------*/
package speculoos.jndi.mappers;

import java.util.Map;

import javax.naming.NamingException;
import javax.naming.directory.Attributes;

import speculoos.core.MapperException;
import speculoos.jndi.AddMapper;

/**
 * Implementation of add mapper.
 * 
 * @author jeff
 * @version $Id$
 */
public class AddMapperImpl extends JNDIMapper implements AddMapper {

	/*
	 * transformer from object to maps
	 */
	private AttributesFromInput attributesFromInput;

	/**
	 * Creates a named mapper.
	 * 
	 * @param value
	 *            the name of this mapper.
	 */
	public AddMapperImpl(String value) {
		super(value);
	}

	/**
	 * Default constructor for creating anonymous mapper.
	 * 
	 */
	public AddMapperImpl() {
		this("add");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.norsys.mapper.jndi.AddMapper#add(java.util.Map, java.util.Map)
	 */
	public Object add(Object input, Map env) throws MapperException {
		return map(input,env);
	}

	/* (non-Javadoc)
	 * @see speculoos.jndi.mappers.JNDIMapper#doMap(java.lang.Object, java.util.Map)
	 */
	protected Object doMap(Object input, Map env) throws MapperException {
		if(attributesFromInput == null)
			throw new MapperException(getName()+": Undefined transformation mapper");
		try {
			if (log.isDebugEnabled())
				log.debug("[" + getName() + "] adding root=" + _root
						+ " with attributes=" + input);
			/* transform */
			Attributes attrs = (Attributes) attributesFromInput.map(input,env);
			directory.createSubcontext(_root, attrs);
			if (log.isDebugEnabled())
				log.debug("[" + getName() + "] done");
		} catch (NamingException e) {
			throw new MapperException("Exception in JNDI adding "
					+ e.getLocalizedMessage(), e);
		}
		return input;
	}

	/**
	 * Defines the type of input objects for this add mapper and 
	 * the mapping rules.
	 * 
	 * @param cls
	 */
	public void setOutput(Class cls,Map map){
		this.attributesFromInput = AttributesFromInput.create(cls,map);
	}
}
